/*
 * Project name:
     RTC3 click (Demonstration of mikroBUS RTC3 click board)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
     20150401:
     - initial release (DM);
 * Description:
      This project is simple demonstration how to read/write date and time from
      BQ32000DR RTC (real-time clock).
      Date and time are read from the RTC every 10 milisecond and printed on TFT.
      Pressing D0 button will write default time and date to RTC module.
      When button is released RTC will start counting.
      User can change default time and date in main function.
 * Test configuration:
     MCU:             PIC32MX795F512L
                      http://www.microchip.com/wwwproducts/Devices.aspx?product=PIC32MX795F512L
     Dev.Board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL 80.0000 MHz
     ext. modules:    RTC3 click board click
                      http://www.mikroe.com/click/rtc3/
     SW:              mikroC PRO for PIC32
                      http://www.mikroe.com/mikroc/pic32/
 * NOTES:
     - Turn on TFT backlight switch SW11.1
     - Place pull down on RA15 (PORTA&C three state switch), and turn on PORTA&C BUTTON PRESS LEVEL
     - RTC3 click board with BQ32000DR is placed to the mikroBUS socket 1.
       Program uses I2C2 perpheral to comunicate with RTC chip.
 */

#include "Resources.h"

// RTC Definitions
#define RTC_ADDR  0xD0

// Module variables
unsigned char hours, minutes, seconds, day, week, month, year;    // Global date/time variables
char oldstate = 0;

char charge[3] = {0x08, 0x20, 0x45};

/**************************************************************************************************
* Draw Screen
**************************************************************************************************/
void DrawScr(){
  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Pen(CL_Black, 1);
  TFT_Line(20, 220, 300, 220);
  TFT_LIne(20,  46, 300,  46);
  TFT_Set_Font(HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("RTC3  click  TEST", 75, 14);
  TFT_Set_Font(Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("EasyPIC Fusion v7", 19, 223);
  TFT_Set_Font(Verdana12x13_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("www.mikroe.com", 200, 223);
  TFT_Set_Font(TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
}

/**************************************************************************************************
* BQ32000DR Functions
**************************************************************************************************/

/**************************************************************************************************
* Read data from RTC BQ32000DR
* input : addres of RTC register
* output: value of of RTC register
**************************************************************************************************/
unsigned char RTC_Read(unsigned char addr){
  unsigned char value;

  I2C2_Start();                   // Issue start signal
  I2C2_Write(RTC_ADDR);           // Address BQ32000DR, see BQ32000DR datasheet
  I2C2_Write(addr);               // Start from address 2
  I2C2_Start();                   // Issue repeated start signal
  I2C2_Write(RTC_ADDR);           // Address BQ32000DR for reading R/W=1

  value = I2C2_Read(0);           // Read seconds byte
  I2C2_Stop();                    // Issue stop signal

  return value;
}

/**************************************************************************************************
* Write data from BQ32000DR
* input : addres of RTC register, value of of RTC register
**************************************************************************************************/
void RTC_Write(unsigned char addr, unsigned char value){
  I2C2_Start();         // Issue start signal
  I2C2_Write(RTC_ADDR); // Address BQ32000DR
  I2C2_Write(addr);     // Start from address
  I2C2_Write(value);    // Write value to RTC register
  I2C2_Stop();          // Issue stop signal
}

/**************************************************************************************************
* Read time from RTC BQ32000DR
* input : pointer to variables where RTC data will be stored
* output: variables with RTC data
**************************************************************************************************/
void Read_Time(unsigned char *p_hours, unsigned char *p_minutes, unsigned char *p_seconds,
               unsigned char *p_day, unsigned char *p_week, unsigned char *p_month, unsigned char *p_year){
  I2C2_Start();                // Issue start signal
  I2C2_Write(RTC_ADDR);        // Address BQ32000DR, see BQ32000DR datasheet
  I2C2_Write(0);               // Start from address 0
  I2C2_Restart();              // Issue repeated start signal
  I2C2_Write(RTC_ADDR + 1);    // Address BQ32000DR for reading R/W=1

  *p_seconds = I2C2_Read(0);   // Read seconds byte
  *p_minutes = I2C2_Read(0);   // Read minutes byte
  *p_hours = I2C2_Read(0);     // Read hours byte
  *p_week =I2C2_Read(0);
  *p_day =I2C2_Read(0);
  *p_month =I2C2_Read(0);
  *p_year =I2C2_Read(1);

  I2C2_Stop();                 // Issue stop signal
}

/**************************************************************************************************
* Write time to RTC BQ32000DR
* input : variables with RTC data
**************************************************************************************************/
void Write_Time(unsigned char c_hours, unsigned char c_minutes, unsigned char c_seconds,
                unsigned char c_day, unsigned char c_week, unsigned char c_month, unsigned char c_year){
   I2C2_Start();                        // issue start signal
   I2C2_Write(RTC_ADDR);                // address BQ32000DR
   I2C2_Write(0);                       // start from word at address (REG0)
   I2C2_Write(0x80);                    // write $80 to REG0. (pause counter + 0 sec)

   I2C2_Write(c_minutes);               // write 0 to minutes word to (REG1)
   I2C2_Write(c_hours);                 // write 17 to hours word (24-hours mode)(REG2)
   I2C2_Write(c_week);                  // write 2 - Monday (REG3)
   I2C2_Write(c_day);                   // write 4 to date word (REG4)
   I2C2_Write(c_month);                 // write 5 (May) to month word (REG5)
   I2C2_Write(c_year);                  // write 01 to year word (REG6)
   I2C2_Stop();                         // issue stop signal

   I2C2_Start();                        // issue start signal
   I2C2_Write(RTC_ADDR);                // address BQ32000DR
   I2C2_Write(0);                       // start from word at address 0
   I2C2_Write(0 | c_seconds);           // write 0 to REG0 (enable counting + 0 sec)
   I2C2_Stop();                         // issue stop signal
}

/**************************************************************************************************
* Show on the LCD display
* input : variables with RTC data
**************************************************************************************************/
void Show_Time(){
char loop;
static char oldseconds = 0xFF;
static char oldminutes = 0xFF;
static char oldhours   = 0xFF;
static char oldday     = 0xFF;
static char oldmonth   = 0xFF;
static char old__year  = 0xFF;
static char oldwday    = 0xFF;

  char *txt;

  seconds  =  ((seconds & 0x70) >> 4)*10 + (seconds & 0x0F);
  minutes  =  ((minutes & 0xF0) >> 4)*10 + (minutes & 0x0F);
  hours    =  ((hours & 0x30) >> 4)*10 + (hours & 0x0F);
  week     =  (week & 0x07);
  day      =  ((day & 0xF0) >> 4)*10 + (day & 0x0F);
  month    =  ((month & 0x10) >> 4)*10 + (month & 0x0F);
  year     =  ((year & 0xF0)>>4)*10+(year & 0x0F);

  if (week != oldwday){
    TFT_Set_Font(Tahoma25x25_Bold, CL_WHITE, FO_HORIZONTAL);
    for (loop = 0; loop < 2; loop++){
      switch (oldwday){
        case 1 : {
                  TFT_Write_Text("Sun", 120, 80);
                  break;
                 }
        case 2 : {
                  TFT_Write_Text("Mon", 120, 80);
                  break;
                 }
        case 3 : {
                  TFT_Write_Text("Tue", 120, 80);
                  break;
                 }
        case 4 : {
                  TFT_Write_Text("Wed", 120, 80);
                  break;
                 }
        case 5 : {
                  TFT_Write_Text("Thu", 120, 80);
                  break;
                 }
        case 6 : {
                  TFT_Write_Text("Fri", 120, 80);
                  break;
                 }
        case 7 : {
                  TFT_Write_Text("Sat", 120, 80);
                  break;
                 }
      }
      oldwday = week;
      TFT_Set_Font(Tahoma25x25_Bold, CL_BLACK, FO_HORIZONTAL);
    }
  }

  if (day != oldday){
    TFT_Set_Font(Tahoma25x25_Bold, CL_WHITE, FO_HORIZONTAL);
    TFT_Write_Char((oldday / 10)   + 48, 100, 120);    // Print tens digit of day variable
    TFT_Write_Char((oldday % 10)   + 48, 115, 120);    // Print oness digit of day variable
    TFT_Set_Font(Tahoma25x25_Bold, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Char((day / 10)   + 48, 100, 120);    // Print tens digit of day variable
    TFT_Write_Char((day % 10)   + 48, 115, 120);    // Print oness digit of day variable
    oldday = day;
  }
  if (month != oldmonth){
    TFT_Set_Font(Tahoma25x25_Bold, CL_WHITE, FO_HORIZONTAL);
    TFT_Write_Char((oldmonth / 10) + 48, 140, 120);
    TFT_Write_Char((oldmonth % 10) + 48, 155, 120);
    TFT_Set_Font(Tahoma25x25_Bold, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Char((month / 10) + 48, 140, 120);
    TFT_Write_Char((month % 10) + 48, 155, 120);
    oldmonth = month;
  }
  if (year != old__year){
    TFT_Set_Font(Tahoma25x25_Bold, CL_WHITE, FO_HORIZONTAL);
    TFT_Write_Char((old__year / 10) + 48, 210, 120);
    TFT_Write_Char((old__year % 10) + 48, 225, 120);
    TFT_Set_Font(Tahoma25x25_Bold, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Char('2', 180, 120);
    TFT_Write_Char('0', 195, 120);
    TFT_Write_Char((year / 10) + 48, 210, 120);
    TFT_Write_Char((year % 10) + 48, 225, 120);
    old__year = year;
  }

  if (hours != oldhours){
    TFT_Set_Font(Tahoma25x25_Bold, CL_WHITE, FO_HORIZONTAL);
    TFT_Write_Char((oldhours / 10)   + 48, 100, 160);
    TFT_Write_Char((oldhours % 10)   + 48, 115, 160);
    TFT_Set_Font(Tahoma25x25_Bold, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Char((hours / 10)   + 48, 100, 160);
    TFT_Write_Char((hours % 10)   + 48, 115, 160);
    oldhours = hours;
  }
  if (minutes != oldminutes){
    TFT_Set_Font(Tahoma25x25_Bold, CL_WHITE, FO_HORIZONTAL);
    TFT_Write_Char((oldminutes / 10) + 48, 140, 160);
    TFT_Write_Char((oldminutes % 10) + 48, 155, 160);
    TFT_Set_Font(Tahoma25x25_Bold, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Char((minutes / 10) + 48, 140, 160);
    TFT_Write_Char((minutes % 10) + 48, 155, 160);
    oldminutes = minutes;
  }
  if (seconds != oldseconds){
    TFT_Set_Font(Tahoma25x25_Bold, CL_WHITE, FO_HORIZONTAL);
    TFT_Write_Char((oldseconds / 10) + 48, 180, 160);
    TFT_Write_Char((oldseconds % 10) + 48, 195, 160);
    TFT_Set_Font(Tahoma25x25_Bold, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Char((seconds / 10) + 48, 180, 160);
    TFT_Write_Char((seconds % 10) + 48, 195, 160);
    oldseconds = seconds;
  }
}

/**************************************************************************************************
* MAIN PROGRAM
**************************************************************************************************/
void main(){
  AD1PCFG = 0xFFFF;            // Configure AN pins as digital I/O
  JTAGEN_bit = 0;              // Disable JTAG
  
  // Configure user buttons
  TRISA15_bit = 1;                    // Set RA15 pin as input.
                                      // RA15 is used to write inital data to RTC.

  TFT_Set_Default_Mode();
  TFT_Init_ILI9341_8bit(320,240);     // Initialize TFT
  Delay_ms(500);

  DrawScr();
  
  I2C2_Init(100000);                 // Initialize I2C bus for communication with RTC
  
  TFT_Set_Font(Tahoma25x25_Bold, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("Date:", 30, 120); TFT_Write_Char(45, 130, 120); TFT_Write_Char(45, 170, 120);
  TFT_Write_Text("Time:", 30, 160); TFT_Write_Char(58, 130, 160); TFT_Write_Char(58, 170, 160);
  
  // charging super - capacitor
  I2C2_Start();
  I2C2_Write(RTC_ADDR);
  I2C2_Write(0x08);
  I2C2_Write(0x20);
  I2C2_Write(0x45);
  I2C2_Stop;

  // Start the test examples
  while(1){                                       // Endless loop
    if (Button(&PORTA, 15, 1, 0)) {               // Detect logical one
      oldstate = 1;                               // Update flag
    }
    if (oldstate && Button(&PORTA, 15, 1, 1)) {   // Detect one-to-zero transition
    // Write TIME    16  : 30  : 00  : 1.  Wednesday april  2015
      Write_Time(0x16, 0x30, 0x00, 0x01, 0x04,   0x04, 0x15);
      oldstate = 0;                               // Update flag
    }

    Read_Time(&hours, &minutes, &seconds, &day, &week, &month, &year);
    Show_Time();
    Delay_mS(10);
  }
}